import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_form_builder/flutter_form_builder.dart';
import 'package:flutter_kz_library/src/statics/Statics.dart';
import 'package:intl/intl.dart';

typedef onChangeDateTime = void Function(String value);
typedef onChangeNumeric = void Function(int value);
typedef onChangeText = void Function(String value);
typedef onChangeDropDown = void Function(String value);
typedef onChangeCheckbox = void Function(List<dynamic> value);
typedef onChangeRadio = void Function(String value);
typedef onChangeSwitch = void Function(bool value);

class DatePicker extends StatelessWidget {
  final String attribute;
  final DateFormatType format;
  final bool isRequired;
  final bool enabled;
  final bool readOnly;
  final DateTime initialValue;
  final InputDecoration decoration;
  final onChangeDateTime onChange;

  DatePicker(
      {this.attribute,
      this.format,
      this.isRequired,
      this.enabled,
      this.readOnly,
      this.initialValue,
      this.decoration,
      this.onChange});

  List<FormFieldValidator> validators(bool isRequired) {
    var valid = List<FormFieldValidator>();

    if (isRequired) {
      valid.add(FormBuilderValidators.required());
    }

    return valid;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderDateTimePicker(
      attribute: attribute,
      inputType: InputType.date,
      format: Statics.dateFormat(format: format),
      decoration: decoration,
      validators: validators(isRequired),
      enabled: enabled,
      readOnly: readOnly,
      initialValue: initialValue,
      onChanged: (value) => onChange(value.toString()),
    );
  }
}

class TimePicker extends StatelessWidget {
  final String attribute;
  final bool isRequired;
  final bool enabled;
  final bool readOnly;
  final DateTime initialValue;
  final InputDecoration decoration;
  final onChangeDateTime onChange;

  TimePicker(
      {this.attribute,
      this.isRequired,
      this.enabled,
      this.readOnly,
      this.initialValue,
      this.decoration,
      this.onChange});

  List<FormFieldValidator> validators(bool isRequired) {
    var valid = List<FormFieldValidator>();

    if (isRequired) {
      valid.add(FormBuilderValidators.required());
    }

    return valid;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderDateTimePicker(
      attribute: attribute,
      inputType: InputType.time,
      format: DateFormat('HH:mm'),
      decoration: decoration,
      validators: validators(isRequired),
      enabled: enabled,
      readOnly: readOnly,
      initialValue: initialValue,
      onChanged: (value) => onChange(value.toString()),
    );
  }
}

class DateTimePicker extends StatelessWidget {
  final String attribute;
  final DateFormatType format;
  final bool isRequired;
  final bool enabled;
  final bool readOnly;
  final DateTime initialValue;
  final InputDecoration decoration;
  final onChangeDateTime onChange;

  DateTimePicker(
      {this.attribute,
      this.isRequired,
      this.format,
      this.enabled,
      this.readOnly,
      this.initialValue,
      this.decoration,
      this.onChange});

  List<FormFieldValidator> validators(bool isRequired) {
    var valid = List<FormFieldValidator>();

    if (isRequired) {
      valid.add(FormBuilderValidators.required());
    }

    return valid;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderDateTimePicker(
      attribute: attribute,
      inputType: InputType.both,
      format: Statics.dateFormat(format: format),
      decoration: decoration,
      validators: validators(isRequired),
      enabled: enabled,
      readOnly: readOnly,
      initialValue: initialValue,
      onChanged: (value) => onChange(value.toString()),
    );
  }
}

class NumericField extends StatelessWidget {
  final String attribute;
  final int minValue;
  final int maxValue;
  final bool isRequired;
  final bool enabled;
  final bool readOnly;
  final String initialValue;
  final InputDecoration decoration;
  final onChangeNumeric onChange;

  NumericField(
      {this.attribute,
      this.minValue,
      this.maxValue,
      this.isRequired,
      this.enabled,
      this.readOnly,
      this.initialValue,
      this.decoration,
      this.onChange});

  List<FormFieldValidator> validators(bool isRequired) {
    var valid = List<FormFieldValidator>();
    valid.add(FormBuilderValidators.numeric());
    valid.add(FormBuilderValidators.min(minValue));
    valid.add(FormBuilderValidators.max(maxValue));

    if (isRequired) {
      valid.add(FormBuilderValidators.required());
    }

    return valid;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderTextField(
      attribute: attribute,
      keyboardType: TextInputType.number,
      decoration: decoration,
      validators: validators(isRequired),
      enabled: enabled,
      readOnly: readOnly,
      initialValue: initialValue,
      onChanged: (value) =>
          onChange((value != null && value != '') ? int.parse(value) : 0),
    );
  }
}

class EmailField extends StatelessWidget {
  final String attribute;
  final bool isRequired;
  final bool enabled;
  final bool readOnly;
  final String initialValue;
  final InputDecoration decoration;
  final onChangeText onChange;

  EmailField(
      {this.attribute,
      this.isRequired,
      this.enabled,
      this.readOnly,
      this.initialValue,
      this.decoration,
      this.onChange});

  List<FormFieldValidator> validators(bool isRequired) {
    var valid = List<FormFieldValidator>();

    valid.add(FormBuilderValidators.email());

    if (isRequired) {
      valid.add(FormBuilderValidators.required());
    }

    return valid;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderTextField(
      attribute: attribute,
      keyboardType: TextInputType.emailAddress,
      decoration: decoration,
      validators: validators(isRequired),
      enabled: enabled,
      readOnly: readOnly,
      initialValue: initialValue,
      onChanged: (value) => onChange(value),
    );
  }
}

class DropdownField extends StatelessWidget {
  final String attribute;
  final String hint;
  final bool isRequired;
  final bool readOnly;
  final InputDecoration decoration;
  final List<DropdownMenuItem> data;
  final onChangeDropDown onChange;

  DropdownField(
      {this.attribute,
      this.hint,
      this.isRequired,
      this.readOnly,
      this.decoration,
      this.data,
      this.onChange});

  List<FormFieldValidator> validators(bool isRequired) {
    var valid = List<FormFieldValidator>();

    if (isRequired) {
      valid.add(FormBuilderValidators.required());
    }

    return valid;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderDropdown(
      attribute: attribute,
      decoration: decoration,
      hint: Text(hint),
      validators: validators(isRequired),
      readOnly: readOnly,
      items: data,
      onChanged: (value) => onChange(value),
    );
  }
}

class CheckboxSelect extends StatelessWidget {
  final String attribute;
  final bool readOnly;
  final InputDecoration decoration;
  final List<SelectListModel> data;
  final onChangeCheckbox onChange;

  CheckboxSelect(
      {this.attribute,
      this.data,
      this.readOnly,
      this.decoration,
      this.onChange});

  List<FormBuilderFieldOption> options() {
    var options = List<FormBuilderFieldOption>();

    data.forEach((item) => options.add(FormBuilderFieldOption(
          value: item.itemValue,
          label: item.itemText,
        )));

    return options;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderCheckboxList(
      attribute: attribute,
      decoration: decoration,
      initialValue: [options()[0].value],
      options: options(),
      readOnly: readOnly,
      onChanged: (value) => onChange(value),
    );
  }
}

class FilterChipSelect extends StatelessWidget {
  final String attribute;
  final bool readOnly;
  final InputDecoration decoration;
  final List<SelectListModel> data;
  final onChangeCheckbox onChange;

  FilterChipSelect(
      {this.attribute,
      this.readOnly,
      this.decoration,
      this.data,
      this.onChange});

  List<FormBuilderFieldOption> options() {
    var options = List<FormBuilderFieldOption>();

    data.forEach((item) => options.add(FormBuilderFieldOption(
          value: item.itemValue,
          child: Text(item.itemText),
        )));

    return options;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderFilterChip(
      attribute: attribute,
      initialValue: [options()[0].value],
      options: options(),
      decoration: decoration,
      onChanged: (value) => onChange(value),
    );
  }
}

class RadioSelect extends StatelessWidget {
  final String attribute;
  final bool readOnly;
  final InputDecoration decoration;
  final List<SelectListModel> data;
  final onChangeRadio onChange;

  RadioSelect(
      {this.attribute,
      this.readOnly,
      this.data,
      this.decoration,
      this.onChange});

  List<FormBuilderFieldOption> options() {
    var options = List<FormBuilderFieldOption>();

    data.forEach((item) => options.add(FormBuilderFieldOption(
          value: item.itemValue,
          label: item.itemText,
        )));

    return options;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderRadio(
      attribute: attribute,
      readOnly: readOnly,
      decoration: decoration,
      initialValue: options()[0].value,
      options: options(),
      onChanged: (value) => onChange(value),
    );
  }
}

class ChoiceChipSelect extends StatelessWidget {
  final String attribute;
  final bool readOnly;
  final InputDecoration decoration;
  final List<SelectListModel> data;
  final onChangeRadio onChange;

  ChoiceChipSelect(
      {this.attribute,
      this.readOnly,
      this.decoration,
      this.data,
      this.onChange});

  List<FormBuilderFieldOption> options() {
    var options = List<FormBuilderFieldOption>();

    data.forEach((item) => options.add(FormBuilderFieldOption(
          value: item.itemValue,
          child: Text(item.itemText),
        )));

    return options;
  }

  @override
  Widget build(BuildContext context) {
    return FormBuilderChoiceChip(
      attribute: attribute,
      readOnly: readOnly,
      decoration: decoration,
      options: options(),
      onChanged: (value) => onChange(value),
    );
  }
}

class SwitchField extends StatelessWidget {
  final String attribute;
  final String labelText;
  final bool readOnly;
  final bool initialValue;
  final onChangeSwitch onChange;

  SwitchField(
      {this.attribute,
      this.labelText,
      this.readOnly,
      this.initialValue,
      this.onChange});

  @override
  Widget build(BuildContext context) {
    return FormBuilderSwitch(
      attribute: attribute,
      readOnly: readOnly,
      initialValue: initialValue,
      label: Text(labelText),
      onChanged: (value) => onChange(value),
    );
  }
}

class RateField extends StatelessWidget {
  final String attribute;
  final String labelText;
  final bool readOnly;
  final int initialValue;
  final int maxValue;
  final double iconSize;
  final onChangeNumeric onChange;

  RateField(
      {this.attribute,
      this.readOnly,
      this.labelText,
      this.initialValue,
      this.maxValue,
      this.iconSize,
      this.onChange});

  @override
  Widget build(BuildContext context) {
    return FormBuilderRate(
      attribute: attribute,
      decoration: InputDecoration(labelText: labelText),
      initialValue: initialValue,
      max: maxValue,
      readOnly: readOnly,
      iconSize: iconSize,
      onChanged: (value) => onChange(value),
    );
  }
}
